@@ -4,31 +4,16 @@ from scipy.interpolate import RegularGridInterpolator as RGI
 from scipy.optimize import curve_fit
 from scipy.signal import convolve
 
-from plotastrodata.other_utils import (coord2xy, rel2abs, estimate_rms, trim,
-                                       isdeg, Mfac, Mrot, dot2d, gaussian2d)
+from plotastrodata.coord_utils import coord2xy, rel2abs
+from plotastrodata.matrix_utils import Mfac, Mrot, dot2d
+from plotastrodata.other_utils import (estimate_rms, trim,
+                                       gaussian2d, isdeg,
+                                       RGIxy, RGIxyv, to4dim)
 from plotastrodata.fits_utils import FitsData, data2fits, Jy2K
 from plotastrodata import const_utils as cu
 from plotastrodata.fitting_utils import EmceeCorner
 
 
-def to4dim(data: np.ndarray) -> np.ndarray:
-    """Change a 2D, 3D, or 4D array to a 4D array.
-
-    Args:
-        data (np.ndarray): Input data. 2D, 3D, or 4D.
-
-    Returns:
-        np.ndarray: Output 4D array.
-    """
-    if np.ndim(data) == 2:
-        d = np.array([[data]])
-    elif np.ndim(data) == 3:
-        d = np.array([data])
-    else:
-        d = np.array(data)
-    return d
-
-
 def quadrantmean(data: np.ndarray, x: np.ndarray, y: np.ndarray,
                  quadrants: str = '13') -> tuple[np.ndarray, np.ndarray, np.ndarray]:
     """Take mean between 1st and 3rd (or 2nd and 4th) quadrants.
@@ -62,74 +47,6 @@ def quadrantmean(data: np.ndarray, x: np.ndarray, y: np.ndarray,
     return datanew[ny:, nx:], xnew[nx:], ynew[ny:]
 
 
-def RGIxy(y: np.ndarray, x: np.ndarray, data: np.ndarray,
-          yxnew: tuple[np.ndarray, np.ndarray] | None = None,
-          **kwargs) -> object | np.ndarray:
-    """RGI for x and y at each channel.
-
-    Args:
-        y (np.ndarray): 1D array. Second coordinate.
-        x (np.ndarray): 1D array. First coordinate.
-        data (np.ndarray): 2D, 3D, or 4D array.
-        yxnew (tuple, optional): (ynew, xnew), where ynew and xnew are 1D or 2D arrays. Defaults to None.
-
-    Returns:
-        np.ndarray: The RGI function or the interpolated array.
-    """
-    if not np.ndim(data) in [2, 3, 4]:
-        print('data must be 2D, 3D, or 4D.')
-        return
-
-    _kw = {'bounds_error': False, 'fill_value': np.nan,
-           'method': 'linear'}
-    _kw.update(kwargs)
-    c4d = to4dim(data)
-    c4d[np.isnan(c4d)] = 0
-    f = [[RGI((y, x), c2d, **_kw)
-          for c2d in c3d] for c3d in c4d]
-    if yxnew is None:
-        if len(f) == 1:
-            f = f[0]
-        if len(f) == 1:
-            f = f[0]
-        return f
-    else:
-        return np.squeeze([[f2d(tuple(yxnew)) for f2d in f3d] for f3d in f])
-
-
-def RGIxyv(v: np.ndarray, y: np.ndarray, x: np.ndarray, data: np.ndarray,
-           vyxnew: tuple[np.ndarray, np.ndarray, np.ndarray] | None = None,
-           **kwargs) -> object | np.ndarray:
-    """RGI in the x-y-v space.
-
-    Args:
-        v (np.ndarray): 1D array. Third coordinate.
-        y (np.ndarray): 1D array. Second coordinate.
-        x (np.ndarray): 1D array. First coordinate.
-        data (np.ndarray): 3D or 4D array.
-        vyxnew (tuple, optional): (vnew, ynew, xnew), where vnew, ynew, and xnew are 1D or 2D arrays. Defaults to None.
-
-    Returns:
-        np.ndarray: The RGI function or the interpolated array.
-    """
-    if not np.ndim(data) in [3, 4]:
-        print('data must be 3D or 4D.')
-        return
-
-    _kw = {'bounds_error': False, 'fill_value': np.nan,
-           'method': 'linear'}
-    _kw.update(kwargs)
-    c4d = to4dim(data)
-    c4d[np.isnan(c4d)] = 0
-    f = [RGI((v, y, x), c3d, **_kw) for c3d in c4d]
-    if vyxnew is None:
-        if len(f) == 1:
-            f = f[0]
-        return f
-    else:
-        return np.squeeze([f3d(tuple(vyxnew)) for f3d in f])
-
-
 def filled2d(data: np.ndarray, x: np.ndarray, y: np.ndarray, n: int = 1,
              **kwargs) -> tuple[np.ndarray, np.ndarray, np.ndarray]:
     """Fill 2D data, 1D x, and 1D y by a factor of n using RGI.
@@ -580,7 +497,7 @@ class AstroData():
                 h['BMAJ'] = self.beam_org[0] / 3600
                 h['BMIN'] = self.beam_org[1] / 3600
                 h['BPA'] = self.beam_org[2]
-            else:    
+            else:
                 h['BMAJ'] = self.beam[0] / 3600
                 h['BMIN'] = self.beam[1] / 3600
                 h['BPA'] = self.beam[2]
@@ -750,6 +667,7 @@ class AstroFrame():
                 grid = fd.get_grid(center=d.center[i], dist=self.dist,
                                    restfreq=d.restfreq[i], vsys=self.vsys,
                                    pv=self.pv)
+                d.center[i] = fd.get_center()  # for WCS rotation
                 d.beam[i] = fd.get_beam(dist=self.dist)
                 d.bunit[i] = fd.get_header('BUNIT')
             if d.data[i] is not None:
@@ -788,8 +706,8 @@ class AstroFrame():
                               'CUNIT1': 'DEG',
                               'RESTFREQ': d.restfreq[i]}
                     if None not in d.beam[i]:
-                        header['BMAJ'] = d.beam[i][0] / 3600
-                        header['BMIN'] = d.beam[i][1] / 3600
+                        header['BMAJ'] = d.beam[i][0] / 3600 / self.dist
+                        header['BMIN'] = d.beam[i][1] / 3600 / self.dist
                     d.data[i] = d.data[i] * Jy2K(header=header)
                     d.sigma[i] = d.sigma[i] * Jy2K(header=header)
                 if self.pv and None not in d.beam[i]:
